/*
 *  This program is free software; you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation; either version 2 of the License, or
 *  (at your option) any later version.
 *
 *  This program is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License
 *  along with this program; if not, write to the Free Software
 *  Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
 */
 
#ifndef ENGINE_H
#define ENGINE_H

#include<gtkmm.h>
#include<map>

/** 
 * This class represents the complete day information 
 * from the PCal engine viewpoint.
 */
class PCalDayInfo {
	public:
        int day_num; // -1 if the date state is undefined
                     // 0 for the first day of the period
        bool estimate; // true if day info is just an estimate
                       // false if it is the documented fact
        bool fertile;
        bool pregnancy;
        bool birth;
        bool bad_feel;
        Glib::ustring description;
};

/**
 * The Calendar Record Type
 */
enum PCalRecordType {
  PCAL_TYPE_MENS,
  PCAL_TYPE_PREGNANCY,
  PCAL_TYPE_BIRTH
};

/** 
 * This is a statistics for a single period
 */
class PCalPeriodInfo {
    public:
        Glib::Date startDate;
        Glib::Date endDate;
        int length;
        PCalPeriodInfo() {};
        PCalPeriodInfo(const PCalPeriodInfo& src):
            startDate(src.startDate),
            endDate(src.endDate),
            length(src.length) {};
};

class PCalEngine
{
    protected:
        std::map<Glib::Date, PCalRecordType> start_dates;
        std::map<Glib::Date, Glib::ustring> notes_container; // the notes container

        void recalculate(); // this function is called each time 
                            // the statistics needs to be recalculated
        // these values are from the options
        int bufferDays;
        int badFeelDaysBefore;
        int badFeelDaysAfter;
        int regularMinLength;
        int regularMaxLength;
    
    public:
        bool modified; // true if any changes were done since last save or load
        // the statistics data
        int periodsCount; // regular periods count
        int avgLength;
        int minLength;
        int maxLength;
        int calMethodAccuracy; // 0 - high, 1 - moderate, 2 - low, 3 - no data
    
        PCalEngine(int in_bufferDays,
                   int in_badFeelDaysBefore,
                   int in_badFeelDaysAfter,
                   int in_regularMinLength,
                   int in_regularMaxLength);
        void changeOptions(int in_bufferDays,
                   int in_badFeelDaysBefore,
                   int in_badFeelDaysAfter,
                   int in_regularMinLength,
                   int in_regularMaxLength);
        void addStartDate(const Glib::Date date);
        /// Record the pregnancy
        /// \param date is the first day of the last menstruation
        void addPregnancyDate(const Glib::Date date);
        void addBirthDate(const Glib::Date date);
        void removeDateRecord(const Glib::Date date);
        void clear();
        /// Unicode file name is expected
        bool loadFromFile(const Glib::ustring fname);
        /// Unicode file name is expected
        bool saveToFile(const Glib::ustring fname);
        /// Import data from the foreign file
        /// Currenly a single data format is supported - date list(0)
        /// Unicode file name is expected
        /// \returns true if the import was successful
        bool importFromFile(const Glib::ustring fname, int format = 0);
        /// Export data to the foreign file
        /// Currenly a single data format is supported - date list(0)
        /// Unicode file name is expected
        /// \returns true if the export was successful
        bool exportToFile(const Glib::ustring fname, int format = 0);
        /// Get the complete information about the date specified
        /// \param date - the date
        /// \returns - information for the date specified
        PCalDayInfo getDayInfo(const Glib::Date date);
        /// Get the statistics for all recorded periods
        /// \returns - the vector of the information records 
        ///            for each preriod
        const std::vector<PCalPeriodInfo> getPeriodsStats();
        /// Get the last menstruation starting date
        /// \param result - the container where the result will be stored to
        /// \returns true if the date was found
        bool getLastMenstruationDate(Glib::Date& result);
        
        /// get the date note content
        bool existsDateNote(const Glib::Date date);
        bool getDateNote(const Glib::Date date, Glib::ustring& note);
        void setDateNote(const Glib::Date date, const Glib::ustring note);
        void deleteDateNote(const Glib::Date date);
        const std::map<Glib::Date, Glib::ustring>& getDateNotes();
};

#endif // ENGINE_H
